from .client import AwsRoute53Client
from ..error.zonecontrol import ZoneControlError
from ..error.awsresponse import AwsResponseError

class AwsZoneManager:
    def __init__(self, zone, aws_client = None):
        if(aws_client == None):
            aws_client = AwsRoute53Client.create()
        self.__client = aws_client
        self.__zone = zone

    def getClient(self):
        return self.__client

    def getZoneId(self):
        response = self.__client.list_hosted_zones()

        name = response.getZoneIdForName(self.__zone)
        if name == None:
            raise ZoneControlError()
        return name

    def getValofRecord(self, record):
        response = self.__client.list_resource_record_sets(
            HostedZoneId=self.getZoneId(),
            StartRecordName=record.fqdn(),
            StartRecordType=record.type()
        )

        if response.isOK():
            recordSets = response.getRecordSetsByName(record.fqdn())
            return  None if len(recordSets) == 0  else recordSets[0]['ResourceRecords'][0]['Value']
        else:
            raise AwsResponseError()

    def upsertRecord(self, record):
        batchedChanges = {
            'Action': 'UPSERT',
            'ResourceRecordSet': {
                'Name': record.fqdn(),
                'Type': record.type(),
                'ResourceRecords': [{'Value': "\"{}\"".format(record.val())}],
                'TTL': record.ttl()
            }
        }
        response = self.__client.change_resource_record_sets(
            HostedZoneId=self.getZoneId(),
            ChangeBatch={
                'Changes': [ batchedChanges ]
            })
        return response



from .lookup_error import LookupError

import dns.resolver
import socket

class LocalClient:
    def getHostByName(self, query_string):
        try:
            return socket.gethostbyname(query_string)
        except socket.gaierror as e:
            raise LookupError(e)

    def resolveTXTRecord(self, record_string):
        try:
            answers = []
            for answer in dns.resolver.resolve(record_string,"TXT").rrset:
                answers.append(answer.to_text().strip('"'))
            return answers
        except dns.resolver.NXDOMAIN as e:
            raise LookupError(e)



from .local_client import LocalClient


class LocalResolver:
    def __init__(self, domain, client = None):
        if (client == None):
            client = LocalClient()
        self.__domain = domain
        self.__client = client

    def fetchTXTRecords(self, query_string):
        answers = self.__client.resolveTXTRecord(query_string)
        massagedAnswers = []
        for answer in answers:
            massagedAnswers.append(answer  + "." + self.__domain)
        return massagedAnswers
        
    def queryForIP(self, query_string):
        return self.__client.getHostByName(query_string)


class ListHostedZonesResponse:
    def __init__(self, response):
        self.__response = response

    def getZoneIdForName(self, name):
        for zone in self.__response['HostedZones']:
            if zone['Name'] == name:
                return zone['Id']
        return None


from .record import Record

class TXTRecord:
    def __init__(self, name, ip=None):
        self.__record = Record("TXT", name, ip)

    def __getattr__(self, attr):
        if hasattr(self.__record, attr):
            def wrapper(*args, **kw):
                return getattr(self.__record, attr)(*args, **kw)
            return wrapper
        raise AttributeError(attr)



from .response.list_hosted_zones import ListHostedZonesResponse
from .response.list_resource_record_sets import ListResourceRecordSetsResponse

class AwsRoute53Client:
    def __init__(self, client):
        self.__client = client

    def list_hosted_zones(self, *args, **kwargs):
        response = self.__client.list_hosted_zones(*args, **kwargs)
        return ListHostedZonesResponse(response)

    def list_resource_record_sets(self, *args, **kwargs):
        response = self.__client.list_resource_record_sets(*args, **kwargs)
        return ListResourceRecordSetsResponse(response)

    def change_resource_record_sets(self, *args, **kwargs):
        self.__client.change_resource_record_sets(*args, **kwargs)

    @staticmethod
    def create(access_key_id = None, secret_access_key = None):
        import boto3

        if (access_key_id == None or secret_access_key == None):
            client = boto3.client('route53')
        else:
            client = boto3.client('route53', access_key_id, secret_access_key)
        return AwsRoute53Client(client)


class Record:
    def __init__(self, type, name, ip=None, ttl = 60):
        self.__type = type
        self.__name = name
        self.__ip = ip
        self.__ttl = ttl

    def fqdn(self):
        return self.__name

    def val(self):
        return self.__ip

    def type(self):
        return self.__type

    def ttl(self):
        return self.__ttl


class ListResourceRecordSetsResponse:
    def __init__(self, response):
        self.__response = response

    def isOK(self):
        if self.__response['ResponseMetadata']:
            metadata = self.__response['ResponseMetadata']
            return metadata['HTTPStatusCode'] == 200
        return False

    def getRecordSetsByName(self, name):
        recordSets = []

        def getRecordSetVal(rs):
            return rs['Name'] == name

        if self.isOK():
            recordSets = self.__response['ResourceRecordSets']
            recordSets = list(filter(getRecordSetVal, recordSets))

        return recordSets
